/*
 * THIS PROGRAM IS GENERATED UNDER LICENSE FROM NEC CORPORATION.
 */
/*global document, jQuery, systemcommon, log */

/**
 * @fileoverview 会員情報管理機能(MemberInfo)の各画面と各イベントの紐付けを定義するファイル。
 */

/**
 * 会員情報管理機能(MemberInfo)の名前空間。
 * @namespace
 */
var memberinfo = {};

(function ($) {
	"use strict";

	/**
	 * 会員情報管理機能(MemberInfo)の画面IDとPagecontainer Widgetイベントを紐付けた連想配列。
	 * @memberof memberinfo
	 * @type {Object.<string, Array.<string>>}
	 */
	memberinfo.pageEventArray = {
		InsertMemberInfo: ["pagecontainerbeforeshow"],
		InsertConf: []
	};

	/**
	 * 画面に紐付いたViewModelクラスを取得します。
	 * @memberof memberinfo
	 * @function
	 * @param {string} pageId 画面ID
	 * @return {memberinfo.InsertMemberInfoViewModel|memberinfo.InsertConfViewModel|null}
	 */
	memberinfo.createViewModel = function (pageId) {
		switch (pageId) {
			case "InsertMemberInfo":
				return new memberinfo.InsertMemberInfoViewModel();
			case "InsertConf":
				return new memberinfo.InsertConfViewModel();
			default:
				return null;
		}
	};

	/**
	 * 遷移先画面IDを取得します。
	 * @memberof memberinfo
	 * @function
	 * @param {Object} data データオブジェクト
	 * @return {string} 遷移先画面ID
	 */
	memberinfo.getToPageId = function (data) {
		/**
		 * 遷移先画面ID
		 * @type {string}
		 */
		var toPageId = "";
		if (data.toPage && typeof data.toPage === "object") {
			toPageId = data.toPage[0].id;
		} else if (data.toPage && typeof data.toPage === "string") {
			toPageId = data.toPage;
		}
		return toPageId;
	};

	$(document)
		// 各論理画面のpagecontainerbeforeshowを実行します。
		.on("pagecontainerbeforeshow", function (event, data) {
			/**
			 * 遷移先画面ID
			 * @type {string}
			 */
			var pageId = memberinfo.getToPageId(data);

			/**
			 * 画面に紐付いたイベントの配列
			 * @type {Array.<string>}
			 */
			var eventArray = memberinfo.pageEventArray[pageId];
			if (!eventArray || eventArray.indexOf("pagecontainerbeforeshow") < 0) {
				return;
			}

			/**
			 * 画面に紐付いたViewModelのインスタンス
			 * @type {memberinfo.InsertMemberInfoViewModel|memberinfo.InsertConfViewModel|null}
			 */
			var target = memberinfo.createViewModel(pageId);
			if (!target) {
				log.error("Illegal screen ID(" + pageId + ").");
				return;
			}
			if (!target.ev_pagecontainerbeforeshow) {
				log.warn("'pagecontainerbeforeshow' event definition is missing for " + pageId + "ViewModel. Please check the design information.");
				return;
			}
			target.ev_pagecontainerbeforeshow(event, data);
		});

	$(document)
		// 会員情報登録画面(InsertMemberInfo) ページ初期処理時に各イベントをバインディングします。
		.on("pagecreate", "#InsertMemberInfo", function (event) {

			// 会員情報登録画面(InsertMemberInfo) 確認画面を表示する
			$(document).on("tap", "#InsertMemberInfo #next", function (event) {
				/**
				 * InsertMemberInfoのViewModelのインスタンスを生成する。
				 * @type {memberinfo.InsertMemberInfoViewModel}
				 */
				var target = new memberinfo.InsertMemberInfoViewModel();
				target.ev_tapNext(event);
			});
		});

	$(document)
		// 確認画面(InsertConf) ページ初期処理時に各イベントをバインディングします。
		.on("pagecreate", "#InsertConf", function (event) {

			// 確認画面(InsertConf) 会員情報登録画面に戻る。
			$(document).on("tap", "#InsertConf #cancel", function (event) {
				/**
				 * InsertConfのViewModelのインスタンスを生成する。
				 * @type {memberinfo.InsertConfViewModel}
				 */
				var target = new memberinfo.InsertConfViewModel();
				target.ev_tapCancel(event);
			});

			// 確認画面(InsertConf) ユーザが入力した値で登録処理を行う。
			$(document).on("tap", "#InsertConf #insert", function (event) {
				/**
				 * InsertConfのViewModelのインスタンスを生成する。
				 * @type {memberinfo.InsertConfViewModel}
				 */
				var target = new memberinfo.InsertConfViewModel();
				target.ev_tapInsert(event);
			});
		});

}(jQuery));
