/*
 * COPYRIGHT (C) 2014 NEC CORPORATION
 *
 * ALL RIGHTS RESERVED BY NEC CORPORATION,  THIS PROGRAM
 * MUST BE USED SOLELY FOR THE PURPOSE FOR WHICH IT WAS
 * FURNISHED BY NEC CORPORATION,  NO PART OF THIS PROGRAM
 * MAY BE REPRODUCED OR DISCLOSED TO OTHERS,  IN ANY FORM
 * WITHOUT THE PRIOR WRITTEN PERMISSION OF NEC CORPORATION.
 *
 * NEC CONFIDENTIAL AND PROPRIETARY
 *
 */
/*global $, log4javascript */

/**
 * @fileoverview システム共通処理を定義するファイル。
 */

/**
 * システム共通の名前空間。
 * @namespace
 */
var systemcommon = {};

/**
 * SystemCommonDao がアクセスするサーバ名を指定します。<br>
 * TODO: 値を実装してください。
 * @constant
 */
systemcommon.ServerName = "";

/**
 * SystemCommonDao がアクセスするコンテキストルートを指定します。<br>
 * TODO: 値を実装してください。
 * @constant
 */
systemcommon.ContextRoot = "";

/**
 * 全てのリスナを差し込むかどうかを判定するフラグです。<br>
 * trueの場合、リスナが定義されている場合に実行します。<br>
 * falseの場合、リスナが定義されているいないに関わらず実行しません。
 * @constant
 */
systemcommon.AllListenerExecFlg = true;

/**
 * 全てのAOPを実行するかどうかを判定するフラグです。<br>
 * trueの場合、AOPが定義されている場合に実行します。<br>
 * falseの場合、AOPが定義されているいないに関わらず実行しません。
 * @constant
 */
systemcommon.AllAOPExecFlg = true;

/**
 * 本クラスでは、システム共通のログに関する処理を提供します。
 * @memberof systemcommon
 * @class システム共通のログに関する処理を提供するクラス。
 * @constructor
 */
systemcommon.Log = function () {
	"use strict";
};

/**
 * log4javascript のログ出力レベルのオブジェクトを返却します。<br>
 * 引数に何も指定しなかった場合は、「INFO」のオブジェクトを返却します。<br>
 * 引数の level には、下記の種類の指定が可能です。
 * <ul>
 *     <li>ALL ： 全て出力する</li>
 *     <li>OFF ： 全て出力しない</li>
 *     <li>FATAL ： 致命的レベルのみ出力する</li>
 *     <li>ERROR ： エラーレベル以上を出力する</li>
 *     <li>WARN ： 警告レベル以上を出力する</li>
 *     <li>INFO ： 情報レベル以上を出力する</li>
 *     <li>DEBUG ： デバッグレベル以上を出力する</li>
 *     <li>TRACE ： トレースレベル以上を出力する</li>
 * </ul>
 * なお、ログ出力用変数({@link log})はデフォルトでは DEBUG レベルを適用しています。
 * @memberof systemcommon.Log
 * @function
 * @param {string} level ログ出力レベルのキー
 * @returns {log4javascript.Level} 指定した log4javascript.Level オブジェクト
 * @see http://log4javascript.org/docs/manual.html#levels
 */
systemcommon.Log.logLevel = (function () {
	"use strict";

	/**
	 * ログレベル保持変数
	 * @type {Object.<string, log4javascript.Level>}
	 */
	var logLevelMap = {
		"ALL"   : log4javascript.Level.ALL,
		"OFF"   : log4javascript.Level.OFF,
		"FATAL" : log4javascript.Level.FATAL,
		"ERROR" : log4javascript.Level.ERROR,
		"WARN"  : log4javascript.Level.WARN,
		"INFO"  : log4javascript.Level.INFO,
		"DEBUG" : log4javascript.Level.DEBUG,
		"TRACE" : log4javascript.Level.TRACE
	};

	return function (level) {
		return logLevelMap[level] || log4javascript.Level.INFO;
	};
}());

/**
 * log4javascript のログアペンダーのインスタンスを返却します。<br>
 * 引数に何も指定しなかった場合は、「BrowserConsoleAppender」のインスタンスを返却します。<br>
 * 引数の append には、下記の種類の指定が可能です。
 * <ul>
 *     <li>Alert ： AlertAppender</li>
 *     <li>Ajax ： AjaxAppender</li>
 *     <li>PopUp ： PopUpAppender</li>
 *     <li>InPage ： InPageAppender</li>
 *     <li>BrowserConsole ： BrowserConsoleAppender</li>
 * </ul>
 * ただし、AjaxAppender は引数にファイルのパス指定が必要なため、適切なパスを指定してからご利用いただくようお願いします。<br>
 * ※デフォルトではコメントアウトしています。<br>
 * <br>
 * なお、ログ出力用変数({@link log})はデフォルトでは BrowserConsoleAppender を適用しています。
 * @memberof systemcommon.Log
 * @function
 * @param {string=} append ログアペンダーのキー
 * @returns {log4javascript.Appender} 指定した log4javascript のアペンダーインスタンス
 * @see http://log4javascript.org/docs/manual.html#appenders
 */
systemcommon.Log.appender = (function () {
	"use strict";

	/**
	 * ログアペンダ保持変数
	 * @type {Object.<string, log4javascript.Appender>}
	 */
	var appenderMap = {
		"Alert"          : new log4javascript.AlertAppender(),
//		"Ajax"           : new log4javascript.AjaxAppender(url),
		"PopUp"          : new log4javascript.PopUpAppender(),
		"InPage"         : new log4javascript.InPageAppender(),
		"BrowserConsole" : new log4javascript.BrowserConsoleAppender()
	};

	return function (append) {
		return appenderMap[append] || new log4javascript.BrowserConsoleAppender();
	};
}());

/**
 * ログ出力用変数です。<br>
 * ログレベルにはデフォルトで DEBUG レベルを指定しています。<br>
 * ログアペンダーにはデフォルトで BrowserConsoleAppender を指定しています。<br>
 * ログのレイアウトはデフォルトでは下記のように指定しています。<br>
 * <p><i>%d{yyyy-MM-dd HH:mm:ss} %-5p %c - %m{1}%n</i></p>
 * それぞれの意味については、log4javascriptのマニュアルを参照してください。<br>
 * 必要に応じて適切なログレベル、ログアペンダー、レイアウトに変更してください。
 * @see http://log4javascript.org/docs/manual.html#patternlayout
 * @type {log4javascript.Logger}
 */
var log = log4javascript.getLogger("logger");
(function () {
	"use strict";

	log.setLevel(systemcommon.Log.logLevel("DEBUG"));

	/**
	 * ログアペンダ
	 * @type {log4javascript.Appender}
	 */
	var appender = systemcommon.Log.appender();

	/**
	 * ログフォーマット
	 * @type {log4javascript.PatternLayout}
	 */
	var layout = new log4javascript.PatternLayout("%d{yyyy-MM-dd HH:mm:ss} %-5p %c - %m{1}%n");
	appender.setLayout(layout);
	log.addAppender(appender);
}());

/**
 * 本クラスは、コンストラクタ内で共通データを定義しオブジェクトとして返却します。<br>
 * 共通データにはバージョンを設定し、サーババージョンとの整合性を取ります。
 * @memberof systemcommon
 * @class 共通データを定義するクラス。
 * @constructor
 * @returns {Object} 共通データ
 * @see systemcommon.SystemCommonDao#createData
 */
systemcommon.CommonData = function () {
	"use strict";

	/**
	 * 共通データ
	 * @type {Object}
	 */
	var commonData = {};
	commonData.version = "1.00";
	return commonData;
};

/**
 * システム共通のユーティリティメソッドを提供します。
 * @memberof systemcommon
 */
systemcommon.SystemCommonUtil = (function () {
	"use strict";

	/**
	 * システム共通ユーティリティ
	 * @type {systemcommon.SystemCommonUtil}
	 */
	var instance = null;

	/**
	 * @constructor
	 */
	systemcommon.SystemCommonUtil = function () {
	};

	instance = new systemcommon.SystemCommonUtil();
	return instance;
}());

/**
 * リスナを対象メソッドの前後に追加します。<br>
 * デフォルトで対象とするメソッドは以下の通りです。
 * <ul>
 *     <li>各画面イベントに紐付いたイベントメソッド</li>
 *     <li>Dao 呼び出しメソッド</li>
 *     <li>非同期通信成功時のメソッド</li>
 *     <li>非同期通信失敗時のメソッド</li>
 * </ul>
 * @memberof systemcommon.SystemCommonUtil
 * @function
 * @param {Array} listenerArray リスナ配列
 * @param {Function} func 対象のFunction
 * @returns {Function} func に各リスナを追加した Function
 */
systemcommon.SystemCommonUtil.listen = function (listenerArray, func) {
	"use strict";
	if (!systemcommon.AllListenerExecFlg
			|| listenerArray === undefined
			|| listenerArray.length === 0) {
		return func;
	}

	return function () {
		/**
		 * リスナ配列のサイズ
		 * @type {number}
		 */
		var listenerNum;
		listenerNum = listenerArray.length;

		/**
		 * ループ変数
		 * @type {number}
		 */
		var i;
		for (i = 0; i < listenerNum; i += 1) {
			listenerArray[i].beforeExec();
		}

		func.apply(func, arguments);

		/**
		 * ループ変数
		 * @type {number}
		 */
		var j;
		for (j = listenerNum - 1; -1 < j; j -= 1) {
			listenerArray[j].afterExec();
		}
	};
};

/**
 * システム共通のエラーコールバックメソッドです。
 * @memberof systemcommon.SystemCommonUtil
 * @param {Object} xhr XMLHttpRequestオブジェクト
 * @param {string} status リクエスト結果を表す文字列
 * @param {Object} errorThrown 例外オブジェクト
 */
systemcommon.SystemCommonUtil.errorCallback = function (xhr, status, errorThrown) {
	"use strict";
	// 失敗時はエラーログを出力する。
	log.error(xhr.status + ":" + errorThrown);
};

/**
 * 本クラスでは、システム共通のDaoに関する処理を定義します。
 * @memberof systemcommon
 * @class システム共通のDaoクラス。
 * @constructor
 */
systemcommon.SystemCommonDao = function () {
	"use strict";
};

/**
 * SystemCommonDao がアクセスする URL を作成します。<br>
 * {@link systemcommon.ServerName} と {@link systemcommon.ContextRoot}、引数で渡される URL のキーを連結します。
 * @memberof systemcommon.SystemCommonDao#
 * @function
 * @param {string} urlKey URLのキー
 * @returns {string} URL
 */
systemcommon.SystemCommonDao.prototype.createUrl = function (urlKey) {
	"use strict";

	/**
	 * URL
	 * @type {string}
	 */
	var url = "";
	url += systemcommon.ServerName;
	url += "/";
	url += systemcommon.ContextRoot;
	url += "/";
	url += urlKey;
	url += "/";
	return url;
};

/**
 * SystemCommonDao がサーバへ渡すデータの作成処理です。<br>
 * オブジェクトに URL とサーバへ渡すデータ(JSON)を保持し、返却します。<br>
 * サーバへ渡すデータは、{@link systemcommon.CommonData} と DaoData をJSONにパースしたものです。
 * @memberof systemcommon.SystemCommonDao#
 * @function
 * @param {string} url URL
 * @param {Object} daoData DaoData
 * @returns {Object} 入力データ
 */
systemcommon.SystemCommonDao.prototype.createData = function (url, daoData) {
	"use strict";

	/**
	 * 入力データ
	 * @type {Object}
	 */
	var data = {};
	data.url = url;

	/**
	 * 共通データ
	 * @type {Object}
	 */
	var commonData = new systemcommon.CommonData();

	/**
	 * サーバへ送信するJSONデータの一時保存用変数
	 * @type {Object}
	 */
	var jsonData = {};
	jsonData.inputData = daoData;
	if (commonData !== undefined) {
		jsonData.commonData = commonData;
	}
	data.jsonData = JSON.stringify(jsonData);

	return data;
};

/**
 * SystemCommonDao の ajax メソッドを利用した非同期通信を行います。<br>
 * {@link systemcommon.SystemCommonDao#callAjax} メソッドに対してリスナを追加し実行します。
 * @memberof systemcommon.SystemCommonDao#
 * @function
 * @param {string} url URL
 * @param {Object} daoData DaoData
 * @param {Function} callback 成功時のコールバック関数
 * @param {Function} errorCallback 失敗時のコールバック関数
 */
systemcommon.SystemCommonDao.prototype.invoke = function (url, daoData, callback, errorCallback) {
	"use strict";

	/**
	 * 本サンプルでは、コールバックするのみ
	 */

	var xhr = {};
    callback(daoData, 0, xhr);
};

/**
 * 任意の Dao クラスを用いて非同期通信を行う場合に利用します。<br>
 * {@link systemcommon.SystemCommonDao#callAjax} メソッドに対してリスナを追加し実行します。
 * @memberof systemcommon.SystemCommonDao#
 * @function
 * @param {string} url URL
 * @param {Object} daoData DaoData
 * @param {Function} callback 成功時のコールバック関数
 * @param {Function} errorCallback 失敗時のコールバック関数
 * @param {Object} customDao 任意のDaoクラス
 */
systemcommon.SystemCommonDao.prototype.customInvoke = function (url, daoData, callback, errorCallback, customDao) {
	"use strict";

	/**
	 * リスナ処理を追加したFunction
	 * @type {Function}
	 */
	var execFunc = systemcommon.SystemCommonUtil.listen(systemcommon.DaoListenerArray, this.callAjax);
	execFunc(url, daoData, callback, errorCallback, customDao);
};

/**
 * 非同期通信を行う際の事前の処理を行います。<br>
 * まず、URLの作成、及び入力データの作成を行います。<br>
 * 次に、引数で渡された成功時のコールバック関数と失敗時のコールバック関数に、それぞれシステム共通のリスナを差し込みます。<br>
 * 最後に非同期通信を行うメソッド({@link systemcommon.SystemCommonDao#ajax})を呼び出します。
 * @memberof systemcommon.SystemCommonDao
 * @function
 * @param {string} urlKey URLのキー
 * @param {Object} daoData DaoData
 * @param {Function} callback 成功時のコールバック関数
 * @param {Function} errorCallback 失敗時のコールバック関数
 * @param {Object} dao 利用するDaoクラス
 */
systemcommon.SystemCommonDao.prototype.callAjax = function (urlKey, daoData, callback, errorCallback, dao) {
	"use strict";

	/**
	 * URL
	 * @type {string}
	 */
	var url = dao.createUrl(urlKey);

	/**
	 * 送信データ
	 * @type {Object}
	 */
	var inputData = dao.createData(url, daoData);
	dao.ajax(inputData, systemcommon.SystemCommonUtil.listen(systemcommon.DaoSuccessListenerArray, callback),
			systemcommon.SystemCommonUtil.listen(systemcommon.DaoErrorListenerArray, errorCallback));
};

/**
 * HTTP(Ajax)の非同期通信を行います。<br>
 * 本メソッドが利用する inputData オブジェクトには、url プロパティと jsonData プロパティが必要です。<br>
 * url プロパティには SystemCommonDao がアクセスするURLを、<br>
 * jsonData プロパティには サーバへ渡すデータを JSON にパースしたものが必要です。
 * @memberof systemcommon.SystemCommonDao#
 * @function
 * @param {Object} inputData 入力データ
 * @param {Function} callback 成功時のコールバック関数
 * @param {Function} errorCallback 失敗時のコールバック関数
 */
systemcommon.SystemCommonDao.prototype.ajax = function (inputData, callback, errorCallback) {
	"use strict";
	$.ajax({
		processData: false,
		dataType: "json",
		contentType: "application/json",
		type: "POST",
		url: inputData.url,
		data: inputData.jsonData,
		success: callback,
		error: errorCallback
	});
};

/**
 * システム共通で各画面イベントに対応するイベントメソッドの前後に差し込むリスナを定義します。<br>
 * リスナが差し込まれている場合、各画面イベントに対応するイベントメソッドが実行される前後に beforeExec メソッド、afterExec メソッドが呼び出されます。
 * @memberof systemcommon
 */
systemcommon.ScreenEventListener = {
	/**
	 * イベントメソッドが呼び出される前に、システム共通で実行する処理を実装します。
	 * @memberof systemcommon.ScreenEventListener
	 * @function
	 */
	beforeExec: function () {
		"use strict";
	},

	/**
	 * イベントメソッドが呼び出された後に、システム共通で実行する処理を実装します。
	 * @memberof systemcommon.ScreenEventListener
	 * @function
	 */
	afterExec: function () {
		"use strict";
	}
};

/**
 * システム共通で Dao 呼び出し処理の前後に差し込むリスナを定義します。<br>
 * リスナが差し込まれている場合、Dao 呼び出し処理が実行される前後に beforeExec メソッド、afterExec メソッドが呼び出されます。
 * @memberof systemcommon
 */
systemcommon.DaoListener = {
	/**
	 * Dao 呼び出し処理が呼び出される前に、システム共通で実行する処理を実装します。
	 * @memberof systemcommon.DaoListener
	 * @function
	 */
	beforeExec: function () {
		"use strict";
	},

	/**
	 * Dao 呼び出し処理が呼び出された後に、システム共通で実行する処理を実装します。
	 * @memberof systemcommon.DaoListener
	 * @function
	 */
	afterExec: function () {
		"use strict";
	}
};

/**
 * システム共通で非同期通信成功時の処理に差し込むリスナを定義します。<br>
 * リスナが差し込まれている場合、非同期通信成功時の処理が実行される前後に beforeExec メソッド、afterExec メソッドが呼び出されます。
 * @memberof systemcommon
 */
systemcommon.DaoSuccessListener = {
	/**
	 * 非同期通信成功時の処理が呼び出される前に、システム共通で実行する処理を実装します。
	 * @memberof systemcommon.DaoSuccessListener
	 * @function
	 */
	beforeExec: function () {
		"use strict";
	},

	/**
	 * 非同期通信成功時の処理が呼び出された後に、システム共通で実行する処理を実装します。
	 * @memberof systemcommon.DaoSuccessListener
	 * @function
	 */
	afterExec: function () {
		"use strict";
	}
};

/**
 * システム共通で非同期通信失敗時の処理に差し込むリスナを定義します。<br>
 * リスナが差し込まれている場合、非同期通信失敗時の処理が実行される前後に beforeExec メソッド、afterExec メソッドが呼び出されます。
 * @memberof systemcommon
 */
systemcommon.DaoErrorListener = {
	/**
	 * 非同期通信失敗時の処理が呼び出される前に、システム共通で実行する処理を実装します。
	 * @memberof systemcommon.DaoErrorListener
	 * @function
	 */
	beforeExec: function () {
		"use strict";
	},

	/**
	 * 非同期通信失敗時の処理が呼び出された後に、システム共通で実行する処理を実装します。
	 * @memberof systemcommon.DaoErrorListener
	 * @function
	 */
	afterExec: function () {
		"use strict";
	}
};

/**
 * システム共通で各画面イベントに対応するイベントメソッドに差し込むリスナ配列です。
 * @memberof systemcommon
 */
systemcommon.ScreenEventListenerArray = [systemcommon.ScreenEventListener];

/**
 * システム共通で Dao 呼び出し処理に差し込むリスナ配列です。
 * @memberof systemcommon
 */
systemcommon.DaoListenerArray = [systemcommon.DaoListener];

/**
 * システム共通で非同期通信成功時の処理に差し込むリスナ配列です。
 * @memberof systemcommon
 */
systemcommon.DaoSuccessListenerArray = [systemcommon.DaoSuccessListener];

/**
 * システム共通で非同期通信失敗時の処理に差し込むリスナ配列です。
 * @memberof systemcommon
 */
systemcommon.DaoErrorListenerArray = [systemcommon.DaoErrorListener];
